@@ -44,16 +44,12 @@ module Data.Functor.Bind.Class (
   ) where
 
 import Data.Semigroup
-import Data.Tagged
 import Control.Applicative
 import Control.Applicative.Backwards
 import Control.Applicative.Lift
 import Control.Arrow
 import Control.Category
 import Control.Monad (ap)
-#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ < 707
-import Control.Monad.Instances ()
-#endif
 import Control.Monad.Trans.Cont
 import Control.Monad.Trans.Error
 import Control.Monad.Trans.Except
@@ -83,8 +79,13 @@ import Data.Functor.Product as Functor
 import Data.Functor.Reverse
 import Data.Functor.Extend
 import Data.List.NonEmpty
+import Data.Orphans ()
 import Prelude hiding (id, (.))
 
+#if MIN_VERSION_base(4,4,0)
+import Data.Complex
+#endif
+
 #ifdef MIN_VERSION_containers
 import qualified Data.IntMap as IntMap
 import Data.IntMap (IntMap)
@@ -94,6 +95,20 @@ import Data.Sequence (Seq)
 import Data.Tree (Tree)
 #endif
 
+#ifdef MIN_VERSION_tagged
+import Data.Tagged
+#endif
+
+#if defined(MIN_VERSION_tagged) || MIN_VERSION_base(4,7,0)
+import Data.Proxy
+#endif
+
+#ifdef MIN_VERSION_unordered_containers
+import Data.Hashable
+import Data.HashMap.Lazy (HashMap)
+import qualified Data.HashMap.Lazy as HashMap
+#endif
+
 #ifdef MIN_VERSION_comonad
 import Control.Comonad
 import Control.Comonad.Trans.Env
@@ -138,6 +153,20 @@ class Functor f => Apply f where
   (<.) :: f a -> f b -> f a
   a <. b = const <$> a <.> b
 
+#ifdef MIN_VERSION_tagged
+instance Apply (Tagged a) where
+  (<.>) = (<*>)
+  (<.) = (<*)
+  (.>) = (*>)
+#endif
+
+#if defined(MIN_VERSION_tagged) || MIN_VERSION_base(4,7,0)
+instance Apply Proxy where
+  (<.>) = (<*>)
+  (<.) = (<*)
+  (.>) = (*>)
+#endif
+
 instance Apply f => Apply (Backwards f) where
   Backwards f <.> Backwards a = Backwards (flip id <$> a <.> f)
 
@@ -235,6 +264,11 @@ instance Arrow a => Apply (WrappedArrow a b) where
   (<. ) = (<* )
   ( .>) = ( *>)
 
+#if MIN_VERSION_base(4,4,0)
+instance Apply Complex where
+  (a :+ b) <.> (c :+ d) = a c :+ b d
+#endif
+
 #ifdef MIN_VERSION_containers
 -- | A Map is not 'Applicative', but it is an instance of 'Apply'
 instance Ord k => Apply (Map k) where
@@ -257,6 +291,12 @@ instance Apply Tree where
   ( .>) = ( *>)
 #endif
 
+#ifdef MIN_VERSION_unordered_containers
+-- | A 'HashMap' is not 'Applicative', but it is an instance of 'Apply'
+instance (Hashable k, Eq k) => Apply (HashMap k) where
+  (<.>) = HashMap.intersectionWith id
+#endif
+
 -- MaybeT is _not_ the same as Compose f Maybe
 instance (Functor m, Monad m) => Apply (MaybeT m) where
   (<.>) = apDefault
@@ -415,6 +455,18 @@ apDefault f x = f >>- \f' -> f' <$> x
 instance Semigroup m => Bind ((,)m) where
   ~(m, a) >>- f = let (n, b) = f a in (m <> n, b)
 
+#ifdef MIN_VERSION_tagged
+instance Bind (Tagged a) where
+  Tagged a >>- f = f a
+  join (Tagged a) = a
+#endif
+
+#if defined(MIN_VERSION_tagged) || MIN_VERSION_base(4,7,0)
+instance Bind Proxy where
+  _ >>- _ = Proxy
+  join _ = Proxy
+#endif
+
 instance Bind (Either a) where
   Left a  >>- _ = Left a
   Right a >>- f = f a
@@ -516,6 +568,14 @@ instance ArrowApply a => Bind (WrappedArrow a b) where
   (>>-) = (>>=)
 -}
 
+#if MIN_VERSION_base(4,4,0)
+instance Bind Complex where
+  (a :+ b) >>- f = a' :+ b' where
+    a' :+ _  = f a
+    _  :+ b' = f b
+  {-# INLINE (>>-) #-}
+#endif
+
 #ifdef MIN_VERSION_containers
 -- | A 'Map' is not a 'Monad', but it is an instance of 'Bind'
 instance Ord k => Bind (Map k) where
@@ -532,6 +592,17 @@ instance Bind Tree where
   (>>-) = (>>=)
 #endif
 
+#ifdef MIN_VERSION_unordered_containers
+-- | A 'HashMap' is not a 'Monad', but it is an instance of 'Bind'
+instance (Hashable k, Eq k) => Bind (HashMap k) where
+  -- this is needlessly painful
+  m >>- f = HashMap.fromList $ do
+    (k, a) <- HashMap.toList m
+    case HashMap.lookup k (f a) of
+      Just b -> [(k,b)]
+      Nothing -> []
+#endif
+
 infixl 4 <<.>>, <<., .>>
 
 class Bifunctor p => Biapply p where
@@ -579,9 +650,11 @@ instance Biapply Const where
   Const f <<.>> Const x = Const (f x)
   {-# INLINE (<<.>>) #-}
 
+#ifdef MIN_VERSION_tagged
 instance Biapply Tagged where
   Tagged f <<.>> Tagged x = Tagged (f x)
   {-# INLINE (<<.>>) #-}
+#endif
 
 instance (Biapply p, Apply f, Apply g) => Biapply (Biff p f g) where
   Biff fg <<.>> Biff xy = Biff (bimap (<.>) (<.>) fg <<.>> xy)
