@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE CPP, TypeOperators #-}
 
 #ifndef MIN_VERSION_semigroups
 #define MIN_VERSION_semigroups(x,y,z) 0
@@ -40,7 +40,15 @@ import Data.Functor.Product as Functor
 import Data.Functor.Reverse
 import Data.Functor.Sum
 import Data.List.NonEmpty (NonEmpty(..))
+
+#if MIN_VERSION_base(4,4,0)
+import Data.Complex
+#endif
+
+#ifdef MIN_VERSION_tagged
 import Data.Tagged
+#endif
+
 import Data.Traversable.Instances ()
 
 #ifdef MIN_VERSION_containers
@@ -48,14 +56,45 @@ import Data.Tree
 #endif
 
 import Data.Semigroup hiding (Product, Sum)
+
+#ifdef MIN_VERSION_generic_deriving
+import Generics.Deriving.Base
+#else
+import GHC.Generics
+#endif
+
 import Prelude hiding (foldr)
 
 class Foldable t => Foldable1 t where
   fold1 :: Semigroup m => t m -> m
   foldMap1 :: Semigroup m => (a -> m) -> t a -> m
+  toNonEmpty :: t a -> NonEmpty a
 
   foldMap1 f = maybe (error "foldMap1") id . getOption . foldMap (Option . Just . f)
   fold1 = foldMap1 id
+  toNonEmpty = foldMap1 (:|[])
+
+instance Foldable1 f => Foldable1 (Rec1 f) where
+  foldMap1 f (Rec1 as) = foldMap1 f as
+
+instance Foldable1 f => Foldable1 (M1 i c f) where
+  foldMap1 f (M1 as) = foldMap1 f as
+
+instance Foldable1 Par1 where
+  foldMap1 f (Par1 a) = f a
+
+instance (Foldable1 f, Foldable1 g) => Foldable1 (f :*: g) where
+  foldMap1 f (as :*: bs) = foldMap1 f as <> foldMap1 f bs
+
+instance (Foldable1 f, Foldable1 g) => Foldable1 (f :+: g) where
+  foldMap1 f (L1 as) = foldMap1 f as
+  foldMap1 f (R1 bs) = foldMap1 f bs
+
+instance Foldable1 V1 where
+  foldMap1 _ v = v `seq` undefined
+
+instance (Foldable1 f, Foldable1 g) => Foldable1 (f :.: g) where
+  foldMap1 f (Comp1 m) = foldMap1 (foldMap1 f) m
 
 class Bifoldable t => Bifoldable1 t where
   bifold1 :: Semigroup m => t m m -> m
@@ -96,9 +135,11 @@ instance Bifoldable1 Const where
   bifoldMap1 f _ (Const a) = f a
   {-# INLINE bifoldMap1 #-}
 
+#ifdef MIN_VERSION_tagged
 instance Bifoldable1 Tagged where
   bifoldMap1 _ g (Tagged b) = g b
   {-# INLINE bifoldMap1 #-}
+#endif
 
 instance (Bifoldable1 p, Foldable1 f, Foldable1 g) => Bifoldable1 (Biff p f g) where
   bifoldMap1 f g = bifoldMap1 (foldMap1 f) (foldMap1 g) . runBiff
@@ -132,6 +173,12 @@ instance Bifoldable1 p => Bifoldable1 (WrappedBifunctor p) where
   bifoldMap1 f g = bifoldMap1 f g . unwrapBifunctor
   {-# INLINE bifoldMap1 #-}
 
+#if MIN_VERSION_base(4,4,0)
+instance Foldable1 Complex where
+  foldMap1 f (a :+ b) = f a <> f b
+  {-# INLINE foldMap1 #-}
+#endif
+
 #ifdef MIN_VERSION_containers
 instance Foldable1 Tree where
   foldMap1 f (Node a []) = f a
@@ -141,6 +188,11 @@ instance Foldable1 Tree where
 instance Foldable1 Identity where
   foldMap1 f = f . runIdentity
 
+#ifdef MIN_VERSION_tagged
+instance Foldable1 (Tagged a) where
+  foldMap1 f (Tagged a) = f a
+#endif
+
 instance Foldable1 m => Foldable1 (IdentityT m) where
   foldMap1 f = foldMap1 f . runIdentityT
 
@@ -167,6 +219,7 @@ instance (Foldable1 f, Foldable1 g) => Foldable1 (Sum f g) where
 instance Foldable1 NonEmpty where
   foldMap1 f (a :| []) = f a
   foldMap1 f (a :| b : bs) = f a <> foldMap1 f (b :| bs)
+  toNonEmpty = id
 
 instance Foldable1 ((,) a) where
   foldMap1 f (_, x) = f x
