@@ -27,6 +27,7 @@ module Data.Semigroupoid
   , Semi(..)
   ) where
 
+import Control.Applicative
 import Control.Arrow
 import Data.Functor.Bind
 import Data.Semigroup
@@ -42,6 +43,10 @@ import Data.Functor.Extend
 import Control.Comonad
 #endif
 
+#ifdef MIN_VERSION_tagged
+import Data.Tagged (Tagged (..))
+#endif
+
 -- | 'Control.Category.Category' sans 'Control.Category.id'
 class Semigroupoid c where
   o :: c j k -> c i j -> c i k
@@ -83,3 +88,11 @@ instance Semigroup m => Semigroupoid (Semi m) where
 instance Monoid m => Category (Semi m) where
   id = Semi mempty
   Semi m . Semi n = Semi (m `mappend` n)
+
+instance Semigroupoid Const where
+  _ `o` Const a = Const a
+
+#ifdef MIN_VERSION_tagged
+instance Semigroupoid Tagged where
+  Tagged b `o` _ = Tagged b
+#endif
